@@ -7,7 +7,12 @@ from django.apps import apps
 from django.contrib import admin
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.admindocs import utils
-from django.contrib.admindocs.utils import replace_named_groups, replace_unnamed_groups
+from django.contrib.admindocs.utils import (
+    remove_non_capturing_groups,
+    replace_metacharacters,
+    replace_named_groups,
+    replace_unnamed_groups,
+)
 from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
 from django.db import models
 from django.http import Http404
@@ -25,7 +30,7 @@ from django.utils.inspect import (
 from django.utils.translation import gettext as _
 from django.views.generic import TemplateView
 
-from .utils import get_view_name
+from .utils import _is_callback, get_view_name
 
 # Exclude methods starting with these strings from documentation
 MODEL_METHODS_EXCLUDE = ("_", "add_", "delete", "save", "set_")
@@ -161,8 +166,7 @@ class ViewDetailView(BaseAdminDocsView):
 
     @staticmethod
     def _get_view_func(view):
-        urlconf = get_urlconf()
-        if get_resolver(urlconf)._is_callback(view):
+        if _is_callback(view):
             mod, func = get_mod_func(view)
             try:
                 # Separate the module and function, e.g.
@@ -451,7 +455,8 @@ def extract_views_from_urlpatterns(urlpatterns, base="", namespace=None):
     """
     Return a list of views from a list of urlpatterns.
 
-    Each object in the returned list is a two-tuple: (view_func, regex)
+    Each object in the returned list is a four-tuple:
+    (view_func, regex, namespace, name)
     """
     views = []
     for p in urlpatterns:
@@ -483,10 +488,10 @@ def simplify_regex(pattern):
     example, turn "^(?P<sport_slug>\w+)/athletes/(?P<athlete_slug>\w+)/$"
     into "/<sport_slug>/athletes/<athlete_slug>/".
     """
+    pattern = remove_non_capturing_groups(pattern)
     pattern = replace_named_groups(pattern)
     pattern = replace_unnamed_groups(pattern)
-    # clean up any outstanding regex-y characters.
-    pattern = pattern.replace("^", "").replace("$", "").replace("?", "")
+    pattern = replace_metacharacters(pattern)
     if not pattern.startswith("/"):
         pattern = "/" + pattern
     return pattern
