@@ -32,6 +32,7 @@ from .models import (
     BarAccount,
     Book,
     Bookmark,
+    Box,
     Category,
     Chapter,
     ChapterXtra1,
@@ -43,6 +44,7 @@ from .models import (
     Color,
     Color2,
     ComplexSortedPerson,
+    Country,
     CoverLetter,
     CustomArticle,
     CyclicOne,
@@ -125,6 +127,7 @@ from .models import (
     Telegram,
     Thing,
     Topping,
+    Traveler,
     UnchangeableObject,
     UndeletableObject,
     UnorderedObject,
@@ -887,11 +890,21 @@ class RelatedPrepopulatedInline3(admin.TabularInline):
     autocomplete_fields = ["fk", "m2m"]
 
 
+class RelatedPrepopulatedStackedInlineNoFieldsets(admin.StackedInline):
+    model = RelatedPrepopulated
+    extra = 1
+    prepopulated_fields = {
+        "slug1": ["name", "pubdate"],
+        "slug2": ["status"],
+    }
+
+
 class MainPrepopulatedAdmin(admin.ModelAdmin):
     inlines = [
         RelatedPrepopulatedInline1,
         RelatedPrepopulatedInline2,
         RelatedPrepopulatedInline3,
+        RelatedPrepopulatedStackedInlineNoFieldsets,
     ]
     fieldsets = (
         (
@@ -1272,6 +1285,9 @@ site.register(NotReferenced)
 site.register(ExplicitlyProvidedPK, GetFormsetsArgumentCheckingAdmin)
 site.register(ImplicitlyGeneratedPK, GetFormsetsArgumentCheckingAdmin)
 site.register(UserProxy)
+site.register(Box)
+site.register(Country)
+site.register(Traveler)
 
 # Register core models we need in our tests
 site.register(User, UserAdmin)
@@ -1296,6 +1312,31 @@ site7 = admin.AdminSite(name="admin7")
 site7.register(Article, ArticleAdmin2)
 site7.register(Section)
 site7.register(PrePopulatedPost, PrePopulatedPostReadOnlyAdmin)
+site7.register(
+    Pizza,
+    filter_horizontal=["toppings"],
+    fieldsets=(
+        (
+            "Collapsible",
+            {
+                "classes": ["collapse"],
+                "fields": ["toppings"],
+            },
+        ),
+    ),
+)
+site7.register(
+    Question,
+    filter_horizontal=["related_questions"],
+    fieldsets=(
+        (
+            "Not collapsible",
+            {
+                "fields": ["related_questions"],
+            },
+        ),
+    ),
+)
 
 
 # Used to test ModelAdmin.sortable_by and get_sortable_by().
