@@ -14,10 +14,12 @@ class LiveWidgetTests(AdminSeleniumTestCase):
         """
         A roundtrip on a ModelForm doesn't alter the TextField value
         """
+        from selenium.webdriver.common.by import By
+
         article = Article.objects.create(content="\nTst\n")
         self.selenium.get(
             self.live_server_url + reverse("article_form", args=[article.pk])
         )
-        self.selenium.find_element_by_id("submit").click()
+        self.selenium.find_element(By.ID, "submit").click()
         article = Article.objects.get(pk=article.pk)
         self.assertEqual(article.content, "\r\nTst\r\n")
