@@ -5,6 +5,4 @@ class Migration(migrations.Migration):
 
     dependencies = [("migrations", "4_auto")]
 
-    operations = [
-        migrations.RunPython(migrations.RunPython.noop)
-    ]
+    operations = [migrations.RunPython(migrations.RunPython.noop)]
