@@ -75,7 +75,7 @@ except ImportError:
 # datetime.datetime(2011, 9, 1, 13, 20, 30), which translates to
 # 10:20:30 in UTC and 17:20:30 in ICT.
 
-UTC = timezone.utc
+UTC = datetime.timezone.utc
 EAT = timezone.get_fixed_timezone(180)  # Africa/Nairobi
 ICT = timezone.get_fixed_timezone(420)  # Asia/Bangkok
 
@@ -88,6 +88,19 @@ def get_timezones(key):
     return [constructor(key) for constructor in ZONE_CONSTRUCTORS]
 
 
+class UTCAliasTests(SimpleTestCase):
+    def test_alias_deprecation_warning(self):
+        msg = (
+            "The django.utils.timezone.utc alias is deprecated. "
+            "Please update your code to use datetime.timezone.utc instead."
+        )
+        with self.assertRaisesMessage(RemovedInDjango50Warning, msg):
+            timezone.utc
+
+    def test_timezone_module_dir_includes_utc(self):
+        self.assertIn("utc", dir(timezone))
+
+
 @contextmanager
 def override_database_connection_timezone(timezone):
     try:
@@ -203,7 +216,7 @@ class LegacyDatabaseTests(TestCase):
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 23, 20, 20))
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 13, 20, 30))
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 3, 20, 40))
-        result = Event.objects.all().aggregate(Min("dt"), Max("dt"))
+        result = Event.objects.aggregate(Min("dt"), Max("dt"))
         self.assertEqual(
             result,
             {
@@ -499,7 +512,7 @@ class NewDatabaseTests(TestCase):
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 23, 20, 20, tzinfo=EAT))
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT))
         Event.objects.create(dt=datetime.datetime(2011, 9, 1, 3, 20, 40, tzinfo=EAT))
-        result = Event.objects.all().aggregate(Min("dt"), Max("dt"))
+        result = Event.objects.aggregate(Min("dt"), Max("dt"))
         self.assertEqual(
             result,
             {
@@ -651,7 +664,7 @@ class NewDatabaseTests(TestCase):
     @skipIfDBFeature("supports_timezones")
     def test_cursor_execute_accepts_naive_datetime(self):
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT)
-        utc_naive_dt = timezone.make_naive(dt, timezone.utc)
+        utc_naive_dt = timezone.make_naive(dt, datetime.timezone.utc)
         with connection.cursor() as cursor:
             cursor.execute(
                 "INSERT INTO timezones_event (dt) VALUES (%s)", [utc_naive_dt]
@@ -670,7 +683,7 @@ class NewDatabaseTests(TestCase):
     @skipIfDBFeature("supports_timezones")
     def test_cursor_execute_returns_naive_datetime(self):
         dt = datetime.datetime(2011, 9, 1, 13, 20, 30, tzinfo=EAT)
-        utc_naive_dt = timezone.make_naive(dt, timezone.utc)
+        utc_naive_dt = timezone.make_naive(dt, datetime.timezone.utc)
         Event.objects.create(dt=dt)
         with connection.cursor() as cursor:
             cursor.execute(
